import os
import math
import itertools
import pandas as pd
import oommfodt as oo


tol = 1e-20
test_file1 = os.path.join(os.path.dirname(__file__),
                          'test_files', 'file1.odt')
test_file2 = os.path.join(os.path.dirname(__file__),
                          'test_files', 'file2.odt')
test_file3 = os.path.join(os.path.dirname(__file__),
                          'test_files', 'file3.odt')
test_files = [test_file1, test_file2, test_file3]


def test_columns():
    for test_file in test_files:
        columns = oo.columns(test_file, rename=False)
        assert isinstance(columns, list)
        assert all(isinstance(column, str) for column in columns)
        assert all(':' in column for column in columns)

        columns = oo.columns(test_file, rename=True)
        assert isinstance(columns, list)
        assert all(isinstance(column, str) for column in columns)
        assert all(':' not in column for column in columns)


def test_units():
    for test_file in test_files:
        units = oo.units(test_file, rename=False)
        assert isinstance(units, dict)
        assert all(isinstance(unit, str) for unit in units.keys())
        assert all(':' in unit for unit in units.keys())
        assert 'J' in units.values()
        assert '' in units.values()

        units = oo.units(test_file, rename=True)
        assert isinstance(units, dict)
        assert all(isinstance(unit, str) for unit in units.keys())
        assert all(':' not in unit for unit in units.keys())
        assert units['E'] == 'J'
        assert '' in units.values()


def test_data():
    for test_file in test_files:
        data = oo.data(test_file)
        assert isinstance(data, list)
        assert all(isinstance(x, float) for x in itertools.chain(*data))


def test_read():
    for test_file in test_files:
        df = oo.read(test_file, rename=False)
        assert isinstance(df, pd.DataFrame)

        df = oo.read(test_file, rename=True)
        assert isinstance(df, pd.DataFrame)


def test_read_timedriver1():
    df = oo.read(test_file1)
    assert df.shape == (25, 18)

    t = df['t'].values
    assert abs(t[0] - 1e-12) < tol
    assert abs(t[-1] - 25e-12) < tol


def test_read_timedriver2():
    df = oo.read(test_file2)
    assert df.shape == (15, 18)

    t = df['t'].values
    assert abs(t[0] - 1e-12) < tol
    assert abs(t[-1] - 15e-12) < tol


def test_read_mindriver():
    df = oo.read(test_file3)
    assert df.shape == (1, 20)


def test_merge_files():
    df = oo.merge(test_files)
    assert df.shape == (41, 24)
    assert any(math.isnan(x) for x in df['t'].values)



import pytest
import pkg_resources
from .oommfodt import columns, units, data, read, merge


def test():
    return pytest.main(["-v", "--pyargs", "oommfodt"])  # pragma: no cover


__version__ = pkg_resources.get_distribution(__name__).version
__dependencies__ = pkg_resources.require(__name__)



import re
import oommfodt as oo


def test_version():
    assert isinstance(oo.__version__, str)
    assert re.search(r'^\d+.\d+.?\d*$', oo.__version__)


def test_dependencies():
    assert isinstance(oo.__dependencies__, list)
    assert len(oo.__dependencies__) > 0



import re
import pandas as pd

column_dict = {'RungeKuttaEvolve::Totalenergy': 'E',
               'UniformExchange::Energy': 'E_Exchange',
               'DMExchange6Ngbr::Energy': 'E_DMI',
               'DMI_Cnv::Energy': 'E_DMI_Cnv',
               'DMI_T::Energy': 'E_DMI_T',
               'DMI_D2d::Energy': 'E_DMI_D2d',
               'Demag::Energy': 'E_Demag',
               'FixedZeeman::Energy': 'E_Zeeman',
               'UZeeman::Energy': 'E_UZeeman',
               'CubicAnisotropy::Energy': 'E_CubicAnisotropy',
               'TimeDriver::Simulationtime': 't',
               'CGEvolve::Totalenergy': 'E',
               'SpinTEvolve::Totalenergy': 'E',
               'UniaxialAnisotropy::Energy': 'E_UniaxialAnisotropy',
               'UniaxialAnisotropy4::Energy': 'E_UniaxialAnisotropy4',
               'Southampton_UniaxialAnisotropy4::Energy':
               'E_UniaxialAnisotropy'}


def columns(filename, rename=True):
    """Extract column names from an OOMMF .odt file.

    This function extracts the names of columns from an OOMMF .odt
    file and returns it as a list of strings. If rename=True, the
    column names will be renamed to their shorter versions.

    Parameters
    ----------
    filename : str
        Name of an OOMMF .odt file
    rename : bool
        Flag (the default is True) if column names should be renamed
        with their shorter versions.

    Returns
    -------
    list(str)

    Examples
    --------
    Extracting names of columns from an .odt file.

    >>> import os
    >>> import oommfodt as oo
    ...
    >>> odtfile = os.path.join('oommfodt', 'tests', 'test_files', 'file1.odt')
    >>> columns = oo.columns(odtfile)
    >>> type(columns)
    <class 'list'>

    """
    with open(filename) as f:
        lines = f.readlines()

    columns = []
    for line in lines:
        if line.startswith('# Columns:'):
            line_split = re.split(r'Oxs_|Anv_|Southampton_', line)[1:]
            for column in line_split:
                column = re.sub(r'[{}\s]', '', column)
                if rename:
                    if column in column_dict.keys():
                        column = column_dict[column]
                    else:
                        column = column.split(':')[-1]
                columns.append(column)
            break

    return columns


def units(filename, rename=False):
    """Extract units from an OOMMF .odt file.

    This function extracts the units for every column from an OOMMF
    .odt file and returns a dictionary.

    Parameters
    ----------
    filename : str
        Name of an OOMMF .odt file

    Returns
    -------
    dict

    Examples
    --------
    Extracting units for columns from an .odt file.

    >>> import os
    >>> import oommfodt as oo
    ...
    >>> odtfile = os.path.join('oommfodt', 'tests', 'test_files', 'file2.odt')
    >>> units = oo.units(odtfile)
    >>> type(units)
    <class 'dict'>

    """
    with open(filename) as f:
        lines = f.readlines()

    units = []
    for line in lines:
        if line.startswith('# Units:'):
            units = line.split()[2:]
            units = [re.sub(r'[{}]', '', unit) for unit in units]
            break

    return dict(zip(columns(filename, rename=rename), units))


def data(filename):
    """Read data from an OOMMF .odt file.

    This function reads data from an OOMMF .odt file and returns it as
    a list of floats.

    Parameters
    ----------
    filename : str
        Name of an OOMMF .odt file

    Returns
    -------
    list(float)

    Examples
    --------
    Reading data from an .odt file.

    >>> import os
    >>> import oommfodt as oo
    ...
    >>> odtfile = os.path.join('oommfodt', 'tests', 'test_files', 'file3.odt')
    >>> data = oo.data(odtfile)
    >>> type(data)
    <class 'list'>

    """
    with open(filename) as f:
        lines = f.readlines()

    values = []
    for line in lines:
        if not line.startswith("#"):
            values.append(map(float, line.split()))

    return values


def read(filename, rename=True):
    """Read an .odt file and convert it into pandas.DataFrame.

    This function is going to read column names and data from an OOMMF
    .odt file and return a pandas.DataFrame. Because there is no
    appropriate way of adding metadata to the pandas.DataFrame,
    obtaining units from the .odt file is ignored at the moment. If
    rename=True, the column names will be renamed to their shorter
    versions.

    Parameters
    ----------
    filename : str
        Name of an OOMMF .odt file
    rename : bool
        Flag (the default is True) if column names should be renamed
        with their shorter versions.

    Returns
    -------
    pandas.DataFrame

    Examples
    --------
    Reading an .odt file.

    >>> import os
    >>> import oommfodt as oo
    ...
    >>> odtfile = os.path.join('oommfodt', 'tests', 'test_files', 'file1.odt')
    >>> df = oo.read(odtfile)
    >>> type(df)
    <class 'pandas.core.frame.DataFrame'>

    """
    return pd.DataFrame(data(filename),
                        columns=columns(filename, rename=rename))


def merge(odtfiles):
    """Read multiple .odt files and merge them into a single pandas.DataFrame.

    This function takes an iterable of OOMMF .odt files, merges them,
    and returns a single pandas.DataFrame. If there are non-matching
    columns, the missing values will be NaN. If rename=True, the
    column names will be renamed with their shorter versions.

    Parameters
    ----------
    filenames : list(str)
        List of .odt filenames.
    rename : bool
        Flag (the default is True) if column names should be renamed
        with their shorter versions.

    Returns
    -------
    pandas.DataFrame

    Examples
    --------
    Reading and merging odt files.

    >>> import os
    >>> import oommfodt as oo
    ...
    >>> odtfile1 = os.path.join('oommfodt', 'tests', 'test_files', 'file1.odt')
    >>> odtfile2 = os.path.join('oommfodt', 'tests', 'test_files', 'file2.odt')
    >>> odtfile3 = os.path.join('oommfodt', 'tests', 'test_files', 'file3.odt')
    >>> df = oo.merge([odtfile1, odtfile2, odtfile3])
    >>> type(df)
    <class 'pandas.core.frame.DataFrame'>

    """
    dataframes = map(read, odtfiles)
    return pd.concat(dataframes, ignore_index=True, sort=False)
